# Generated by Django 4.0.4 on 2022-05-04 12:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0009_product_slug"),
    ]

    operations = [
        migrations.AlterField(
            model_name="discountcode",
            name="categories",
            field=models.ManyToManyField(
                blank=True,
                related_name="%(class)s_related",
                to="shop.category",
                verbose_name="Categories",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="billing_detail_state",
            field=models.CharField(
                blank=True, max_length=100, verbose_name="State/Region"
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="shipping_detail_state",
            field=models.CharField(
                blank=True, max_length=100, verbose_name="State/Region"
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="related_products",
            field=models.ManyToManyField(
                blank=True, to="shop.product", verbose_name="Related products"
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="upsell_products",
            field=models.ManyToManyField(
                blank=True, to="shop.product", verbose_name="Upsell products"
            ),
        ),
        migrations.AlterField(
            model_name="sale",
            name="categories",
            field=models.ManyToManyField(
                blank=True,
                related_name="%(class)s_related",
                to="shop.category",
                verbose_name="Categories",
            ),
        ),
    ]



from django.db import migrations, models

import cartridge.shop.fields


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0006_auto_20150916_0459"),
    ]

    operations = [
        migrations.AlterField(
            model_name="product",
            name="sku",
            field=cartridge.shop.fields.SKUField(
                max_length=20, null=True, verbose_name="SKU", blank=True
            ),
        ),
        migrations.AlterField(
            model_name="productvariation",
            name="sku",
            field=cartridge.shop.fields.SKUField(
                max_length=20, null=True, verbose_name="SKU", blank=True
            ),
        ),
        migrations.AlterUniqueTogether(
            name="product",
            unique_together={("sku", "site")},
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0008_product_content_model"),
    ]

    operations = [
        migrations.AlterField(
            model_name="product",
            name="slug",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Leave blank to have the URL auto-generated from the title.",
                max_length=2000,
                verbose_name="URL",
            ),
            preserve_default=False,
        ),
    ]



from decimal import Decimal

import mezzanine.core.fields
import mezzanine.utils.models
from django.db import migrations, models
from mezzanine.conf import settings

import cartridge.shop.fields


class Migration(migrations.Migration):

    dependencies = [
        ("pages", "0001_initial"),
        ("sites", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="Cart",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "last_updated",
                    models.DateTimeField(null=True, verbose_name="Last updated"),
                ),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="CartItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "sku",
                    cartridge.shop.fields.SKUField(max_length=20, verbose_name="SKU"),
                ),
                (
                    "description",
                    models.CharField(max_length=2000, verbose_name="Description"),
                ),
                ("quantity", models.IntegerField(default=0, verbose_name="Quantity")),
                (
                    "unit_price",
                    cartridge.shop.fields.MoneyField(
                        decimal_places=2,
                        default=Decimal("0"),
                        max_digits=10,
                        blank=True,
                        null=True,
                        verbose_name="Unit price",
                    ),
                ),
                (
                    "total_price",
                    cartridge.shop.fields.MoneyField(
                        decimal_places=2,
                        default=Decimal("0"),
                        max_digits=10,
                        blank=True,
                        null=True,
                        verbose_name="Total price",
                    ),
                ),
                ("url", models.CharField(max_length=2000)),
                ("image", models.CharField(max_length=200, null=True)),
                (
                    "cart",
                    models.ForeignKey(
                        on_delete=models.CASCADE, related_name="items", to="shop.Cart"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Category",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        on_delete=models.CASCADE,
                        parent_link=True,
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        to="pages.Page",
                    ),
                ),
                (
                    "content",
                    mezzanine.core.fields.RichTextField(verbose_name="Content"),
                ),
                (
                    "featured_image",
                    mezzanine.core.fields.FileField(
                        max_length=255,
                        null=True,
                        verbose_name="Featured Image",
                        blank=True,
                    ),
                ),
                (
                    "price_min",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Minimum price",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "price_max",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Maximum price",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "combined",
                    models.BooleanField(
                        default=True,
                        help_text="If checked, products must match all specified filters, otherwise products can match any specified filter.",
                        verbose_name="Combined",
                    ),
                ),
            ],
            options={
                "ordering": ("_order",),
                "verbose_name": "Product category",
                "verbose_name_plural": "Product categories",
            },
            bases=("pages.page", models.Model),
        ),
        migrations.CreateModel(
            name="DiscountCode",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("title", models.CharField(max_length=100, verbose_name="Title")),
                ("active", models.BooleanField(default=False, verbose_name="Active")),
                (
                    "discount_deduct",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Reduce by amount",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "discount_percent",
                    cartridge.shop.fields.PercentageField(
                        null=True,
                        verbose_name="Reduce by percent",
                        max_digits=5,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "discount_exact",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Reduce to amount",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "valid_from",
                    models.DateTimeField(
                        null=True, verbose_name="Valid from", blank=True
                    ),
                ),
                (
                    "valid_to",
                    models.DateTimeField(
                        null=True, verbose_name="Valid to", blank=True
                    ),
                ),
                (
                    "code",
                    cartridge.shop.fields.DiscountCodeField(
                        unique=True, max_length=20, verbose_name="Code"
                    ),
                ),
                (
                    "min_purchase",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Minimum total purchase",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "free_shipping",
                    models.BooleanField(default=False, verbose_name="Free shipping"),
                ),
                (
                    "uses_remaining",
                    models.IntegerField(
                        help_text="If you wish to limit the number of times a code may be used, set this value. It will be decremented upon each use.",
                        null=True,
                        verbose_name="Uses remaining",
                        blank=True,
                    ),
                ),
                (
                    "categories",
                    models.ManyToManyField(
                        related_name="discountcode_related",
                        verbose_name="Categories",
                        to="shop.Category",
                        blank=True,
                    ),
                ),
            ],
            options={
                "verbose_name": "Discount code",
                "verbose_name_plural": "Discount codes",
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Order",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "billing_detail_first_name",
                    models.CharField(max_length=100, verbose_name="First name"),
                ),
                (
                    "billing_detail_last_name",
                    models.CharField(max_length=100, verbose_name="Last name"),
                ),
                (
                    "billing_detail_street",
                    models.CharField(max_length=100, verbose_name="Street"),
                ),
                (
                    "billing_detail_city",
                    models.CharField(max_length=100, verbose_name="City/Suburb"),
                ),
                (
                    "billing_detail_state",
                    models.CharField(max_length=100, verbose_name="State/Region"),
                ),
                (
                    "billing_detail_postcode",
                    models.CharField(max_length=10, verbose_name="Zip/Postcode"),
                ),
                (
                    "billing_detail_country",
                    models.CharField(max_length=100, verbose_name="Country"),
                ),
                (
                    "billing_detail_phone",
                    models.CharField(max_length=20, verbose_name="Phone"),
                ),
                (
                    "billing_detail_email",
                    models.EmailField(max_length=75, verbose_name="Email"),
                ),
                (
                    "shipping_detail_first_name",
                    models.CharField(max_length=100, verbose_name="First name"),
                ),
                (
                    "shipping_detail_last_name",
                    models.CharField(max_length=100, verbose_name="Last name"),
                ),
                (
                    "shipping_detail_street",
                    models.CharField(max_length=100, verbose_name="Street"),
                ),
                (
                    "shipping_detail_city",
                    models.CharField(max_length=100, verbose_name="City/Suburb"),
                ),
                (
                    "shipping_detail_state",
                    models.CharField(max_length=100, verbose_name="State/Region"),
                ),
                (
                    "shipping_detail_postcode",
                    models.CharField(max_length=10, verbose_name="Zip/Postcode"),
                ),
                (
                    "shipping_detail_country",
                    models.CharField(max_length=100, verbose_name="Country"),
                ),
                (
                    "shipping_detail_phone",
                    models.CharField(max_length=20, verbose_name="Phone"),
                ),
                (
                    "additional_instructions",
                    models.TextField(
                        verbose_name="Additional instructions", blank=True
                    ),
                ),
                (
                    "time",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Time", null=True
                    ),
                ),
                ("key", models.CharField(max_length=40)),
                ("user_id", models.IntegerField(null=True, blank=True)),
                (
                    "shipping_type",
                    models.CharField(
                        max_length=50, verbose_name="Shipping type", blank=True
                    ),
                ),
                (
                    "shipping_total",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Shipping total",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "tax_type",
                    models.CharField(
                        max_length=50, verbose_name="Tax type", blank=True
                    ),
                ),
                (
                    "tax_total",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Tax total",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "item_total",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Item total",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "discount_code",
                    cartridge.shop.fields.DiscountCodeField(
                        max_length=20, verbose_name="Discount code", blank=True
                    ),
                ),
                (
                    "discount_total",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Discount total",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "total",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Order total",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "transaction_id",
                    models.CharField(
                        max_length=255,
                        null=True,
                        verbose_name="Transaction ID",
                        blank=True,
                    ),
                ),
                (
                    "status",
                    models.IntegerField(
                        default=settings.SHOP_ORDER_STATUS_CHOICES[0][0],
                        verbose_name="Status",
                        choices=settings.SHOP_ORDER_STATUS_CHOICES,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=models.CASCADE, editable=False, to="sites.Site"
                    ),
                ),
            ],
            options={
                "ordering": ("-id",),
                "verbose_name": "Order",
                "verbose_name_plural": "Orders",
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="OrderItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "sku",
                    cartridge.shop.fields.SKUField(max_length=20, verbose_name="SKU"),
                ),
                (
                    "description",
                    models.CharField(max_length=2000, verbose_name="Description"),
                ),
                ("quantity", models.IntegerField(default=0, verbose_name="Quantity")),
                (
                    "unit_price",
                    cartridge.shop.fields.MoneyField(
                        decimal_places=2,
                        default=Decimal("0"),
                        max_digits=10,
                        blank=True,
                        null=True,
                        verbose_name="Unit price",
                    ),
                ),
                (
                    "total_price",
                    cartridge.shop.fields.MoneyField(
                        decimal_places=2,
                        default=Decimal("0"),
                        max_digits=10,
                        blank=True,
                        null=True,
                        verbose_name="Total price",
                    ),
                ),
                (
                    "order",
                    models.ForeignKey(
                        on_delete=models.CASCADE, related_name="items", to="shop.Order"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Product",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "keywords_string",
                    models.CharField(max_length=500, editable=False, blank=True),
                ),
                ("rating_count", models.IntegerField(default=0, editable=False)),
                ("rating_sum", models.IntegerField(default=0, editable=False)),
                ("rating_average", models.FloatField(default=0, editable=False)),
                ("title", models.CharField(max_length=500, verbose_name="Title")),
                (
                    "slug",
                    models.CharField(
                        help_text="Leave blank to have the URL auto-generated from the title.",
                        max_length=2000,
                        null=True,
                        verbose_name="URL",
                        blank=True,
                    ),
                ),
                (
                    "_meta_title",
                    models.CharField(
                        help_text="Optional title to be used in the HTML title tag. If left blank, the main title field will be used.",
                        max_length=500,
                        null=True,
                        verbose_name="Title",
                        blank=True,
                    ),
                ),
                (
                    "description",
                    models.TextField(verbose_name="Description", blank=True),
                ),
                (
                    "gen_description",
                    models.BooleanField(
                        default=True,
                        help_text="If checked, the description will be automatically generated from content. Uncheck if you want to manually set a custom description.",
                        verbose_name="Generate description",
                    ),
                ),
                ("created", models.DateTimeField(null=True, editable=False)),
                ("updated", models.DateTimeField(null=True, editable=False)),
                (
                    "status",
                    models.IntegerField(
                        default=2,
                        help_text="With Draft chosen, will only be shown for admin users on the site.",
                        verbose_name="Status",
                        choices=[(1, "Draft"), (2, "Published")],
                    ),
                ),
                (
                    "publish_date",
                    models.DateTimeField(
                        help_text="With Published chosen, won't be shown until this time",
                        null=True,
                        verbose_name="Published from",
                        blank=True,
                    ),
                ),
                (
                    "expiry_date",
                    models.DateTimeField(
                        help_text="With Published chosen, won't be shown after this time",
                        null=True,
                        verbose_name="Expires on",
                        blank=True,
                    ),
                ),
                ("short_url", models.URLField(null=True, blank=True)),
                (
                    "in_sitemap",
                    models.BooleanField(default=True, verbose_name="Show in sitemap"),
                ),
                (
                    "content",
                    mezzanine.core.fields.RichTextField(verbose_name="Content"),
                ),
                (
                    "unit_price",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Unit price",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                ("sale_id", models.IntegerField(null=True)),
                (
                    "sale_price",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Sale price",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "sale_from",
                    models.DateTimeField(
                        null=True, verbose_name="Sale start", blank=True
                    ),
                ),
                (
                    "sale_to",
                    models.DateTimeField(
                        null=True, verbose_name="Sale end", blank=True
                    ),
                ),
                (
                    "sku",
                    cartridge.shop.fields.SKUField(
                        max_length=20,
                        unique=True,
                        null=True,
                        verbose_name="SKU",
                        blank=True,
                    ),
                ),
                (
                    "num_in_stock",
                    models.IntegerField(
                        null=True, verbose_name="Number in stock", blank=True
                    ),
                ),
                (
                    "available",
                    models.BooleanField(
                        default=False, verbose_name="Available for purchase"
                    ),
                ),
                (
                    "image",
                    models.CharField(
                        max_length=100, null=True, verbose_name="Image", blank=True
                    ),
                ),
                (
                    "date_added",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Date added", null=True
                    ),
                ),
                (
                    "categories",
                    models.ManyToManyField(
                        to="shop.Category",
                        verbose_name="Product categories",
                        blank=True,
                    ),
                ),
                (
                    "related_products",
                    models.ManyToManyField(
                        related_name="related_products_rel_+",
                        verbose_name="Related products",
                        to="shop.Product",
                        blank=True,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=models.CASCADE, editable=False, to="sites.Site"
                    ),
                ),
                (
                    "upsell_products",
                    models.ManyToManyField(
                        related_name="upsell_products_rel_+",
                        verbose_name="Upsell products",
                        to="shop.Product",
                        blank=True,
                    ),
                ),
            ],
            options={
                "verbose_name": "Product",
                "verbose_name_plural": "Products",
            },
            bases=(models.Model, mezzanine.utils.models.AdminThumbMixin),
        ),
        migrations.CreateModel(
            name="ProductAction",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("timestamp", models.IntegerField()),
                ("total_cart", models.IntegerField(default=0)),
                ("total_purchase", models.IntegerField(default=0)),
                (
                    "product",
                    models.ForeignKey(
                        on_delete=models.CASCADE,
                        related_name="actions",
                        to="shop.Product",
                    ),
                ),
            ],
            options={},
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="ProductImage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("_order", models.IntegerField(null=True, verbose_name="Order")),
                ("file", models.ImageField(upload_to="product", verbose_name="Image")),
                (
                    "description",
                    models.CharField(
                        max_length=100, verbose_name="Description", blank=True
                    ),
                ),
                (
                    "product",
                    models.ForeignKey(
                        on_delete=models.CASCADE,
                        related_name="images",
                        to="shop.Product",
                    ),
                ),
            ],
            options={
                "ordering": ("_order",),
                "verbose_name": "Image",
                "verbose_name_plural": "Images",
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="ProductOption",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "type",
                    models.IntegerField(
                        verbose_name="Type", choices=[(1, "Size"), (2, "Colour")]
                    ),
                ),
                (
                    "name",
                    cartridge.shop.fields.OptionField(
                        max_length=50, null=True, verbose_name="Name"
                    ),
                ),
            ],
            options={
                "verbose_name": "Product option",
                "verbose_name_plural": "Product options",
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="ProductVariation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "unit_price",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Unit price",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                ("sale_id", models.IntegerField(null=True)),
                (
                    "sale_price",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Sale price",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "sale_from",
                    models.DateTimeField(
                        null=True, verbose_name="Sale start", blank=True
                    ),
                ),
                (
                    "sale_to",
                    models.DateTimeField(
                        null=True, verbose_name="Sale end", blank=True
                    ),
                ),
                (
                    "sku",
                    cartridge.shop.fields.SKUField(
                        max_length=20,
                        unique=True,
                        null=True,
                        verbose_name="SKU",
                        blank=True,
                    ),
                ),
                (
                    "num_in_stock",
                    models.IntegerField(
                        null=True, verbose_name="Number in stock", blank=True
                    ),
                ),
                ("default", models.BooleanField(default=False, verbose_name="Default")),
                (
                    "option1",
                    cartridge.shop.fields.OptionField(
                        max_length=50, null=True, verbose_name="Size"
                    ),
                ),
                (
                    "option2",
                    cartridge.shop.fields.OptionField(
                        max_length=50, null=True, verbose_name="Colour"
                    ),
                ),
                (
                    "image",
                    models.ForeignKey(
                        on_delete=models.CASCADE,
                        verbose_name="Image",
                        blank=True,
                        to="shop.ProductImage",
                        null=True,
                    ),
                ),
                (
                    "product",
                    models.ForeignKey(
                        on_delete=models.CASCADE,
                        related_name="variations",
                        to="shop.Product",
                    ),
                ),
            ],
            options={
                "ordering": ("-default",),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name="Sale",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("title", models.CharField(max_length=100, verbose_name="Title")),
                ("active", models.BooleanField(default=False, verbose_name="Active")),
                (
                    "discount_deduct",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Reduce by amount",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "discount_percent",
                    cartridge.shop.fields.PercentageField(
                        null=True,
                        verbose_name="Reduce by percent",
                        max_digits=5,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "discount_exact",
                    cartridge.shop.fields.MoneyField(
                        null=True,
                        verbose_name="Reduce to amount",
                        max_digits=10,
                        decimal_places=2,
                        blank=True,
                    ),
                ),
                (
                    "valid_from",
                    models.DateTimeField(
                        null=True, verbose_name="Valid from", blank=True
                    ),
                ),
                (
                    "valid_to",
                    models.DateTimeField(
                        null=True, verbose_name="Valid to", blank=True
                    ),
                ),
                (
                    "categories",
                    models.ManyToManyField(
                        related_name="sale_related",
                        verbose_name="Categories",
                        to="shop.Category",
                        blank=True,
                    ),
                ),
                (
                    "products",
                    models.ManyToManyField(
                        to="shop.Product", verbose_name="Products", blank=True
                    ),
                ),
            ],
            options={
                "verbose_name": "Sale",
                "verbose_name_plural": "Sales",
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name="productaction",
            unique_together={("product", "timestamp")},
        ),
        migrations.AddField(
            model_name="discountcode",
            name="products",
            field=models.ManyToManyField(
                to="shop.Product", verbose_name="Products", blank=True
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="category",
            name="options",
            field=models.ManyToManyField(
                related_name="product_options",
                verbose_name="Product options",
                to="shop.ProductOption",
                blank=True,
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="category",
            name="products",
            field=models.ManyToManyField(
                to="shop.Product", verbose_name="Products", blank=True
            ),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name="category",
            name="sale",
            field=models.ForeignKey(
                on_delete=models.CASCADE,
                verbose_name="Sale",
                blank=True,
                to="shop.Sale",
                null=True,
            ),
            preserve_default=True,
        ),
    ]



import csv
import datetime
import os
import shutil

from django.core.management.base import BaseCommand, CommandError
from django.db.utils import IntegrityError
from django.utils.translation import gettext as _
from mezzanine.conf import settings
from mezzanine.core.models import CONTENT_STATUS_PUBLISHED

from cartridge.shop.models import (
    Category,
    Product,
    ProductImage,
    ProductOption,
    ProductVariation,
)

# images get copied from thie directory
LOCAL_IMAGE_DIR = "/tmp/orig"
# images get copied to this directory under STATIC_ROOT
IMAGE_SUFFIXES = [
    ".jpg",
    ".JPG",
    ".jpeg",
    ".JPEG",
    ".tif",
    ".gif",
    ".GIF",
    ".png",
    ".PNG",
]
EMPTY_IMAGE_ENTRIES = ["Please add", "N/A", ""]
DATE_FORMAT = "%Y-%m-%d"
TIME_FORMAT = "%H:%M"

# Here we define what column headings are used in the csv.
TITLE = _("Title")
CONTENT = _("Content")
DESCRIPTION = _("Description")
SKU = _("SKU")
IMAGE = _("Image")
CATEGORY = _("Category")
SUB_CATEGORY = _("Sub-Category")
# SIZE = _("Size")
NUM_IN_STOCK = _("Number in Stock")
UNIT_PRICE = _("Unit Price")
SALE_PRICE = _("Sale Price")
SALE_START_DATE = _("Sale Start Date")
SALE_START_TIME = _("Sale Start Time")
SALE_END_DATE = _("Sale End Date")
SALE_END_TIME = _("Sale End Time")

DATETIME_FORMAT = f"{DATE_FORMAT} {TIME_FORMAT}"
SITE_MEDIA_IMAGE_DIR = _("product")
PRODUCT_IMAGE_DIR = os.path.join(settings.STATIC_ROOT, SITE_MEDIA_IMAGE_DIR)
TYPE_CHOICES = dict()
for id, choice in settings.SHOP_OPTION_TYPE_CHOICES:
    TYPE_CHOICES[choice] = id

fieldnames = [
    TITLE,
    CONTENT,
    DESCRIPTION,
    CATEGORY,
    SUB_CATEGORY,
    SKU,
    IMAGE,
    NUM_IN_STOCK,
    UNIT_PRICE,
    SALE_PRICE,
    SALE_START_DATE,
    SALE_START_TIME,
    SALE_END_DATE,
    SALE_END_TIME,
]
# TODO: Make sure no options conflict with other fieldnames.
fieldnames += list(TYPE_CHOICES.keys())


class Command(BaseCommand):
    help = _("Import/Export products from a csv file.")

    def add_arguments(self, parser):
        parser.add_argument("csv_file")
        parser.add_argument(
            "--import",
            action="store_true",
            dest="import",
            default=False,
            help=_("Import products from csv file."),
        )

        parser.add_argument(
            "--export",
            action="store_true",
            dest="export",
            default=False,
            help=_("Export products from csv file."),
        )

    def handle(self, *args, **options):
        csv_file = options.get("csv_file")
        if options["import"] and options["export"]:
            raise CommandError("can't both import and export")
        if not options["import"] and not options["export"]:
            raise CommandError(_("need to import or export"))
        if options["import"]:
            import_products(csv_file)
        elif options["export"]:
            export_products(csv_file)


def _product_from_row(row):
    product, created = Product.objects.get_or_create(title=row[TITLE])
    product.content = row[CONTENT]
    product.description = row[DESCRIPTION]
    # TODO: set the 2 below from spreadsheet.
    product.status = CONTENT_STATUS_PUBLISHED
    product.available = True
    # TODO: allow arbitrary level/number of categories.
    base_cat, created = Category.objects.get_or_create(title=row[CATEGORY])
    sub_cat, created = Category.objects.get_or_create(
        title=row[SUB_CATEGORY], parent=base_cat
    )
    product.categories.add(sub_cat)
    shop_cat, created = Category.objects.get_or_create(title="Shop")
    product.categories.add(shop_cat)
    return product


def _make_image(image_str, product):
    if image_str in EMPTY_IMAGE_ENTRIES:
        return None
    # try adding various image suffixes, if none given in original filename.
    root, suffix = os.path.splitext(image_str)
    if suffix not in IMAGE_SUFFIXES:
        raise CommandError("INCORRECT SUFFIX: %s" % image_str)
    image_path = os.path.join(LOCAL_IMAGE_DIR, image_str)
    if not os.path.exists(image_path):
        raise CommandError("NO FILE %s" % image_path)
    shutil.copy(image_path, PRODUCT_IMAGE_DIR)
    # shutil.copy(image_path, os.path.join(PRODUCT_IMAGE_DIR, "orig"))
    image, created = ProductImage.objects.get_or_create(
        file="%s" % (os.path.join(SITE_MEDIA_IMAGE_DIR, image_str)),
        description=image_str,  # TODO: handle column for this.
        product=product,
    )
    return image


def _make_date(date_str, time_str):
    date_string = f"{date_str} {time_str}"
    date = datetime.datetime.strptime(date_string, DATETIME_FORMAT)
    return date


def import_products(csv_file):
    print(_("Importing .."))
    # More appropriate for testing.
    # Product.objects.all().delete()
    reader = csv.DictReader(open(csv_file), delimiter=",")
    for row in reader:
        print(row)
        product = _product_from_row(row)
        try:
            variation = ProductVariation.objects.create(
                # strip whitespace
                sku=row[SKU].replace(" ", ""),
                product=product,
            )
        except IntegrityError:
            raise CommandError("Product with SKU exists! sku: %s" % row[SKU])
        if row[NUM_IN_STOCK]:
            variation.num_in_stock = row[NUM_IN_STOCK]
        if row[UNIT_PRICE]:
            variation.unit_price = row[UNIT_PRICE]
        if row[SALE_PRICE]:
            variation.sale_price = row[SALE_PRICE]
        if row[SALE_START_DATE] and row[SALE_START_TIME]:
            variation.sale_from = _make_date(row[SALE_START_DATE], row[SALE_START_TIME])
        if row[SALE_END_DATE] and row[SALE_END_TIME]:
            variation.sale_to = _make_date(row[SALE_END_DATE], row[SALE_END_TIME])
        for option in TYPE_CHOICES:
            if row[option]:
                name = "option%s" % TYPE_CHOICES[option]
                setattr(variation, name, row[option])
                new_option, created = ProductOption.objects.get_or_create(
                    type=TYPE_CHOICES[option], name=row[option]  # TODO: set dynamically
                )
        variation.save()
        image = _make_image(row[IMAGE], product)
        if image:
            variation.image = image
        product.variations.manage_empty()
        product.variations.set_default_images([])
        product.copy_default_variation()
        product.save()

    print("Variations: %s" % ProductVariation.objects.all().count())
    print("Products: %s" % Product.objects.all().count())


def export_products(csv_file):
    print(_("Exporting .."))
    filehandle = open(csv_file, "w")
    writer = csv.DictWriter(filehandle, delimiter=",", fieldnames=fieldnames)
    headers = dict()
    for field in fieldnames:
        headers[field] = field
    writer.writerow(headers)
    for pv in ProductVariation.objects.all():
        row = dict()
        row[TITLE] = pv.product.title
        row[CONTENT] = pv.product.content
        row[DESCRIPTION] = pv.product.description
        row[SKU] = pv.sku
        row[IMAGE] = pv.image
        # TODO: handle multiple categories, and multiple levels of categories
        cat = pv.product.categories.all()[0]
        if cat.parent:
            row[SUB_CATEGORY] = cat.title
            row[CATEGORY] = cat.parent.title
        else:
            row[CATEGORY] = cat.title
            row[SUB_CATEGORY] = ""

        for option in TYPE_CHOICES:
            row[option] = getattr(pv, "option%s" % TYPE_CHOICES[option])

        row[NUM_IN_STOCK] = pv.num_in_stock
        row[UNIT_PRICE] = pv.unit_price
        row[SALE_PRICE] = pv.sale_price
        try:
            row[SALE_START_DATE] = pv.sale_from.strftime(DATE_FORMAT)
            row[SALE_START_TIME] = pv.sale_from.strftime(TIME_FORMAT)
        except AttributeError:
            pass
        try:
            row[SALE_END_DATE] = pv.sale_to.strftime(DATE_FORMAT)
            row[SALE_END_TIME] = pv.sale_to.strftime(TIME_FORMAT)
        except AttributeError:
            pass
        writer.writerow(row)
    filehandle.close()



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0005_auto_20150527_1127"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="key",
            field=models.CharField(max_length=40, db_index=True),
        ),
    ]



import mezzanine.core.managers
import mezzanine.pages.managers
from django.db import migrations, models

import cartridge.shop.managers


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0002_auto_20141227_1331"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order",
            name="billing_detail_email",
            field=models.EmailField(max_length=254, verbose_name="Email"),
        ),
    ]



from django.urls import path, re_path
from mezzanine.conf import settings

from cartridge.shop import views

_slash = "/" if settings.APPEND_SLASH else ""

urlpatterns = [
    re_path(r"^product/(?P<slug>.*)%s$" % _slash, views.product, name="shop_product"),
    path("wishlist%s" % _slash, views.wishlist, name="shop_wishlist"),
    path("cart%s" % _slash, views.cart, name="shop_cart"),
    path("checkout%s" % _slash, views.checkout_steps, name="shop_checkout"),
    path("checkout/complete%s" % _slash, views.complete, name="shop_complete"),
    re_path(
        r"^invoice/(?P<order_id>\d+)%s$" % _slash, views.invoice, name="shop_invoice"
    ),
    re_path(
        r"^invoice/(?P<order_id>\d+)/resend%s$" % _slash,
        views.invoice_resend_email,
        name="shop_invoice_resend",
    ),
]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0007_auto_20150921_2323"),
    ]

    operations = [
        migrations.AddField(
            model_name="product",
            name="content_model",
            field=models.CharField(editable=False, max_length=50, null=True),
        ),
    ]



from mezzanine.conf import settings
from mezzanine.core.translation import TranslatedDisplayable, TranslatedRichText
from modeltranslation.translator import TranslationOptions, translator

from cartridge.shop.models import (
    Category,
    Product,
    ProductImage,
    ProductOption,
    ProductVariation,
)


class TranslatedProduct(TranslatedDisplayable, TranslatedRichText):
    fields = ()


class TranslatedProductImage(TranslationOptions):
    fields = ("description",)


class TranslatedProductOption(TranslationOptions):
    fields = ("name",)


class TranslatedProductVariation(TranslationOptions):
    fields = tuple("option%s" % opt[0] for opt in settings.SHOP_OPTION_TYPE_CHOICES)


class TranslatedCategory(TranslatedRichText):
    fields = ()


translator.register(Product, TranslatedProduct)
translator.register(ProductImage, TranslatedProductImage)
translator.register(ProductOption, TranslatedProductOption)
translator.register(ProductVariation, TranslatedProductVariation)
translator.register(Category, TranslatedCategory)



from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import gettext as _
from mezzanine.conf import settings

from cartridge.shop.checkout import CheckoutError

# Requires Stripe Library Module -- install from pypi.
try:
    import stripe
except ImportError:
    raise ImproperlyConfigured("stripe package must be installed")

try:
    stripe.api_key = settings.STRIPE_API_KEY
except AttributeError:
    raise ImproperlyConfigured(
        "You need to define STRIPE_API_KEY "
        "in your settings module to use the "
        "stripe payment processor."
    )


def process(request, order_form, order):
    """
    Payment handler for the stripe API.
    """
    data = {
        "amount": int((order.total * 100).to_integral()),
        "currency": getattr(settings, "STRIPE_CURRENCY", "usd"),
        "card": {
            "number": request.POST["card_number"].strip(),
            "exp_month": request.POST["card_expiry_month"].strip(),
            "exp_year": request.POST["card_expiry_year"][2:].strip(),
            "cvc": request.POST["card_ccv"].strip(),
            "address_line1": request.POST["billing_detail_street"],
            "address_city": request.POST["billing_detail_city"],
            "address_state": request.POST["billing_detail_state"],
            "address_zip": request.POST["billing_detail_postcode"],
            "country": request.POST["billing_detail_country"],
        },
    }
    try:
        response = stripe.Charge.create(**data)
    except stripe.CardError:
        raise CheckoutError(_("Transaction declined"))
    except Exception as e:
        raise CheckoutError(_("A general error occured: ") + str(e))
    return response.id



import django.db.models.deletion
import mezzanine.core.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="productimage",
            name="_order",
            field=mezzanine.core.fields.OrderField(null=True, verbose_name="Order"),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name="productvariation",
            name="image",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.SET_NULL,
                verbose_name="Image",
                blank=True,
                to="shop.ProductImage",
                null=True,
            ),
            preserve_default=True,
        ),
    ]



import mezzanine.core.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0003_emailfield"),
    ]

    operations = [
        migrations.AlterField(
            model_name="productimage",
            name="file",
            field=mezzanine.core.fields.FileField(verbose_name="Image", max_length=255),
        ),
    ]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("shop", "0004_productimage_file_field"),
    ]

    operations = [
        migrations.AlterField(
            model_name="product",
            name="publish_date",
            field=models.DateTimeField(
                help_text="With Published chosen, won't be shown until this time",
                null=True,
                verbose_name="Published from",
                db_index=True,
                blank=True,
            ),
        ),
    ]



from urllib.error import URLError
from urllib.request import Request, urlopen

from django.core.exceptions import ImproperlyConfigured
from django.utils.http import urlencode
from mezzanine.conf import settings

from cartridge.shop.checkout import CheckoutError

AUTH_NET_LIVE = "https://secure.authorize.net/gateway/transact.dll"
AUTH_NET_TEST = "https://test.authorize.net/gateway/transact.dll"

try:
    AUTH_NET_LOGIN = settings.AUTH_NET_LOGIN
    AUTH_NET_TRANS_KEY = settings.AUTH_NET_TRANS_KEY
except AttributeError:
    raise ImproperlyConfigured(
        "You need to define AUTH_NET_LOGIN and "
        "AUTH_NET_TRANS_KEY in your settings module "
        "to use the authorizenet payment processor."
    )


def process(request, order_form, order):
    """
    Raise cartridge.shop.checkout.CheckoutError("error message") if
    payment is unsuccessful.
    """

    trans = {}
    amount = order.total
    trans["amount"] = amount
    if settings.DEBUG:
        trans["connection"] = AUTH_NET_TEST
    else:
        trans["connection"] = AUTH_NET_LIVE
    trans["authorize_only"] = False
    trans["configuration"] = {
        "x_login": AUTH_NET_LOGIN,
        "x_tran_key": AUTH_NET_TRANS_KEY,
        "x_version": "3.1",
        "x_relay_response": "FALSE",
        "x_test_request": "FALSE",
        "x_delim_data": "TRUE",
        "x_delim_char": "|",
        # could be set to AUTH_ONLY to only authorize but not capture payment
        "x_type": "AUTH_CAPTURE",
        "x_method": "CC",
    }
    data = order_form.cleaned_data
    trans["custBillData"] = {
        "x_first_name": data["billing_detail_first_name"],
        "x_last_name": data["billing_detail_last_name"],
        "x_address": data["billing_detail_street"],
        "x_city": data["billing_detail_city"],
        "x_state": data["billing_detail_state"],
        "x_zip": data["billing_detail_postcode"],
        "x_country": data["billing_detail_country"],
        "x_phone": data["billing_detail_phone"],
        "x_email": data["billing_detail_email"],
    }

    trans["custShipData"] = {
        "x_ship_to_first_name": data["shipping_detail_first_name"],
        "x_ship_to_last_name": data["shipping_detail_last_name"],
        "x_ship_to_address": data["shipping_detail_street"],
        "x_ship_to_city": data["shipping_detail_city"],
        "x_ship_to_state": data["shipping_detail_state"],
        "x_ship_to_zip": data["shipping_detail_postcode"],
        "x_ship_to_country": data["shipping_detail_country"],
    }
    trans["transactionData"] = {
        "x_amount": amount,
        "x_card_num": data["card_number"],
        "x_exp_date": "{month}/{year}".format(
            month=data["card_expiry_month"], year=data["card_expiry_year"]
        ),
        "x_card_code": data["card_ccv"],
        "x_invoice_num": str(order.id),
    }

    part1 = urlencode(trans["configuration"]) + "&"
    part2 = "&" + urlencode(trans["custBillData"])
    part3 = "&" + urlencode(trans["custShipData"])
    trans["postString"] = part1 + urlencode(trans["transactionData"]) + part2 + part3

    request_args = {
        "url": trans["connection"],
        "data": trans["postString"].encode("utf-8"),
    }
    try:
        all_results = urlopen(Request(**request_args)).read()
    except URLError:
        raise CheckoutError("Could not talk to authorize.net payment gateway")

    parsed_results = all_results.decode("utf-8").split(
        trans["configuration"]["x_delim_char"]
    )
    # response and response_reason_codes with their meaning here:
    # http://www.authorize.net/support/merchant/Transaction_Response/
    # Response_Reason_Codes_and_Response_Reason_Text.htm
    # not exactly sure what the reason code is
    response_code = parsed_results[0]
    # reason_code = parsed_results[1]
    # response_reason_code = parsed_results[2]
    # response_text = parsed_results[3]
    # transaction_id = parsed_results[6]
    success = response_code == "1"
    if not success:
        raise CheckoutError("Transaction declined: " + parsed_results[2])
    return parsed_results[6]



"""
This is the local_settings file for Cartridge's docs.
"""

from random import choice

from cartridge.project_template.project_name.settings import *  # noqa: F401, F403

ROOT_URLCONF = "cartridge.project_template.project_name.urls"

# Generate a SECRET_KEY for this build

characters = "abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)"
SECRET_KEY = "".join([choice(characters) for i in range(50)])

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "db.sqlite3",
    }
}



from django.template.defaultfilters import slugify
from mezzanine.conf import settings
from mezzanine.pages.page_processors import processor_for
from mezzanine.utils.views import paginate

from cartridge.shop.models import Category, Product


@processor_for(Category, exact_page=True)
def category_processor(request, page):
    """
    Add paging/sorting to the products for the category.
    """
    settings.clear_cache()
    products = (
        Product.objects.published(for_user=request.user)
        .filter(page.category.filters())
        .distinct()
    )
    sort_options = [
        (slugify(option[0]), option[1]) for option in settings.SHOP_PRODUCT_SORT_OPTIONS
    ]
    sort_by = request.GET.get(
        "sort", sort_options[0][1] if sort_options else "-date_added"
    )
    products = paginate(
        products.order_by(sort_by),
        request.GET.get("page", 1),
        settings.SHOP_PER_PAGE_CATEGORY,
        settings.MAX_PAGING_LINKS,
    )
    products.sort_by = sort_by
    sub_categories = page.category.children.published()
    child_categories = Category.objects.filter(id__in=sub_categories)
    return {"products": products, "child_categories": child_categories}
